datablock StaticShapeData(EmptyShapeData)
{
	shapeFile = "base/data/shapes/empty.dts";
};

function Node(%position)
{
	return new ScriptObject()
	{
		class = Node;
		position = %position;
	};
}

function Node::onAdd(%this)
{
	%this.shape = new StaticShape()
	{
		datablock = EmptyShapeData;
		position = %this.position;
		node = %this;
	};

	%this.linkCount = 0;
}

function Node::onRemove(%this)
{
	%this.shape.delete();

	for (%i = %this.linkCount - 1; %i >= 0; %i--)
	{
		%this.unlink(%this.link[%i]);
	}

	for (%i = %this.linkedByCount - 1; %i >= 0; %i--)
	{
		%this.linkedBy[%i].unlink(%this);
	}
}

function Node::link(%this, %node)
{
	%node = %node.getID();

	if (!%this.isLinked[%node])
	{
		%this.isLinked[%node] = 1;

		%this.link[%this.linkCount] = %node;
		%this.linkCount++;
	}

	if (!%node.isLinkedBy[%this])
	{
		%node.isLinkedBy[%this] = 1;

		%node.linkedBy[%node.linkedByCount] = %this;
		%node.linkedByCount++;
	}

	if (!isObject(%this.lineTo[%node]))
	{
		if (isObject(%node.lineTo[%this]))
		{
			%this.lineTo[%node] = %node.lineTo[%this];
		}
		else
		{
			%this.lineTo[%node] = createShape(CubeGlowShapeData, "0 1 0 1");
			%this.lineTo[%node].transformLine(%this.position, %node.position, 0.01);

			%node.lineTo[%this] = %this.lineTo[%node];
		}
	}

	// test fix
	if (!%node.isLinked[%this])
	{
		%node.link(%this);
	}
}

function Node::unlink(%this, %node)
{
	if (%this.isLinked[%node])
	{
		%this.isLinked[%node] = "";

		for (%i = 0; %i < %this.linkCount; %i++)
		{
			if (%found)
			{
				%this.link[%i] = %this.link[%i + 1];
			}
			else if (%this.link[%i] == %node)
			{
				%found = 1;
				%i--;
			}
		}

		if (%found)
		{
			%this.link[%this.linkCount--] = "";
		}
	}

	if (%node.isLinkedBy[%this])
	{
		%node.isLinkedBy[%this] = "";

		for (%i = 0; %i < %node.linkedByCount; %i++)
		{
			if (%found)
			{
				%node.linkedBy[%i] = %node.linkedBy[%i + 1];
			}
			else if (%node.linkedBy[%i] == %node)
			{
				%found = 1;
				%i--;
			}
		}

		if (%found)
		{
			%node.linkedBy[%node.linkedByCount--] = "";
		}
	}

	if (!%node.isLinked[%this])
	{
		if (isObject(%this.lineTo[%node]))
		{
			%this.lineTo[%node].delete();
		}

		if (isObject(%node.lineTo[%this]))
		{
			%node.lineTo[%this].delete();
		}

		%this.lineTo[%node] = "";
		%node.lineTo[%this] = "";
	}
}

function NodeGroup(%name)
{
	return new ScriptGroup(%name)
	{
		class = NodeGroup;
	};
}

function NodeGroup::import(%this, %fileName)
{
	if (!isFile(%fileName))
	{
		return;
	}

	%this.deleteAll();

	%fp = new FileObject();
	%fp.openForRead(%fileName);

	while (!%fp.isEOF())
	{
		%line = %fp.readLine();

		if (%line $= "neighbors")
		{
			%neighbors = 1;
		}
		else if (%neighbors)
		{
			%a = getWord(%line, 0);
			%b = getWord(%line, 1);

			if (%a < 0 || %b < 0 || %a >= %this.getCount() || %b >= %this.getCount())
			{
				continue;
			}

			%this.getObject(%a).link(%this.getObject(%b));
		}
		else
		{
			%this.add(Node(%line));
		}
	}

	%fp.close();
	%fp.delete();
}

function NodeGroup::findNearest(%this, %position, %range, %masks)
{
	initContainerRadiusSearch(%position, %range, $TypeMasks::StaticShapeObjectType);

	if (%masks)
	{
		%position = vectorAdd(%position, "0 0 1");
	}

	while (isObject(%shape = containerSearchNext()))
	{
		%node = %shape.node;

		if (!isObject(%node) || !%this.isMember(%node))
		{
			continue;
		}

		if (%masks)
		{
			%ray = containerRayCast(%position, vectorAdd(%node.position, "0 0 1"), %masks);

			if (%ray)
			{
				continue;
			}
		}

		return %node;
	}

	return 0;
}